use std::io;

use crate::{
    identifier::Identifier,
    packet::{packet_data::PacketData, raw_packet::RawPacket, Packet, ToPacket},
};
use crate::commands::PacketCommand;

pub mod vec;

#[derive(Debug, Clone, PartialEq, Eq)]
pub enum RawDataTransmission {
    SetupTransmission {
        /// The total size of the data being sent.
        size: u64,
        /// This string is the mime type of the data being sent.  
        /// By default this should be `application/octet-stream`.  
        /// The maximum length of this string is 128 bytes since the mime rfc states the recommended max is 127 bytes.
        mime_type: String,
    },
    Data {
        /// The number of the segment in the sequence of segments being sent.
        sequence_number: u32,
        /// A segment of the data being sent
        data: Vec<u8>,
    },
}

impl PacketCommand for RawDataTransmission {
    fn command(&self) -> u8 {
        match self {
            RawDataTransmission::SetupTransmission { .. } => 0xF0,
            RawDataTransmission::Data { .. } => 0xF1
        }
    }
}

impl ToPacket for RawDataTransmission {
    fn to_packet(
        self,
        src: Identifier,
        dest: Identifier,
        packet_id: Identifier,
        reply_to: Identifier,
    ) -> Packet {
        Packet {
            src,
            dest,
            packet_id,
            reply_to,
            command: 0x20,
            data: PacketData::RawDataTransmission(self.clone()),
        }
    }
}

impl TryFrom<RawPacket> for RawDataTransmission {
    type Error = io::Error;
    fn try_from(raw_packet: RawPacket) -> io::Result<Self> {
        let build_invalid_input_err = |r: &str| Err(io::Error::new(io::ErrorKind::InvalidInput, r));
        match raw_packet.command {
            // setup transmission
            0x00 => {
                let size = u64::from_be_bytes(raw_packet.data[0..8].try_into().unwrap());
                let mime_type = String::from_utf8(raw_packet.data[8..].to_vec()).unwrap();
                Ok(RawDataTransmission::SetupTransmission { size, mime_type })
            }
            // data
            0x01 => {
                let sequence_number = u32::from_be_bytes(raw_packet.data[0..4].try_into().unwrap());
                let data = raw_packet.data[4..].to_vec();
                Ok(RawDataTransmission::Data {
                    sequence_number,
                    data,
                })
            }
            _ => build_invalid_input_err("Invalid command"),
        }
    }
}
