use crate::identifier::Identifier;

mod vec;

#[derive(Debug, Clone, Eq, PartialEq)]
pub enum DataType {
    // Meta
    /// Meta type: Nothing (`0x00`)
    Nothing,
    /// Meta type: Array (`0x01`)
    Array(Vec<DataType>),

    // Basic types
    /// Basic Type: Boolean (`0x10`]
    Boolean(bool),
    /// Basic Type: Slider (`0x11`)
    Number(u64),
    /// Basic Type: Text (`0x12`)
    Text(String),
    /// Basic Type: Identifier (`0x13`)
    Identifier(Identifier),

    // Color
    /// Color Type: RGB (`0x20`)
    RGB(u8, u8, u8),
}

impl DataType {
    pub fn get_data_size(data_type: u8) -> usize {
        match data_type {
            0x00 => 0,
            0x01 => 2,
            0x10 => 1,
            0x11 => 8,
            0x12 => 256,
            0x13 => 4,
            0x20 => 3,
            _ => 0,
        }
    }
}

/// Get a vector of DataTypes.
/// Do **not** put a raw packet in here, **only** a raw datatype vector.
/// Unparsable data automatically gets turned into DataType::Nothing
pub fn get_data_types(raw: Vec<u8>) -> Vec<DataType> {
    let mut data = vec![];
    let mut pieces = vec![];
    let mut byte = raw.iter().peekable();

    while let Some(&c) = byte.next() {
        let mut piece = vec![c];
        let mut len = DataType::get_data_size(c);
        while len > 0 {
            if let Some(b) = byte.next() {
                piece.push(*b);
            }
            len -= 1;
        }
        if piece.len() == DataType::get_data_size(c) + 1 {
            pieces.push(piece);
        } else {
            pieces.push(vec![0x00]);
        }
    }

    for piece in pieces {
        data.push(
            DataType::try_from(piece)
                .unwrap_or(DataType::Nothing)
        )
    }

    data
}
